
#include <stdio.h>
#include <unistd.h>

#include "impulse/util/main.h"
#include "impulse/util/error.h"
#include "impulse/comms/channel.h"

#if 0
/* 
 * Some function declarations
 */
void app_init(impulse_unistring *,
	      impulse_unistring *,
	      impulse_unistring *);
void app_main(impulse_unistring,
	      impulse_unistring,
	      impulse_unistring);

impulse_bool console_event(impulse_event *event,
			   void *data);
impulse_bool transport_event(impulse_transport transport,
			     impulse_transport_event event,
			     void *data);
impulse_bool impulse_service_handler(impulse_channel,
				     impulse_channel_service,
				     impulse_channel_event);


static struct _impulse_unistring service_name = {
  11,11,
  (impulse_unichar[]){
  (impulse_unichar)'T',
  (impulse_unichar)'e',
  (impulse_unichar)'s',
  (impulse_unichar)'t',
  (impulse_unichar)'S',
  (impulse_unichar)'e',
  (impulse_unichar)'r',
  (impulse_unichar)'v',
  (impulse_unichar)'i',
  (impulse_unichar)'c',
  (impulse_unichar)'e',
  (impulse_unichar)'\0'
  }
};

static struct _impulse_channel_service service = 
{
  &service_name,
  1,
  (struct _impulse_channel_version[]) {
    (struct _impulse_channel_version){1,1}
  },
  impulse_service_handler
};
#endif
/*
 * This is where it all starts (as if you didn't know that:)
 */
impulse_int main(impulse_int argc, impulse_char *argv[])
{
  #if 0
  impulse_unistring local = NULL;
  impulse_unistring remote = NULL;
  impulse_unistring protocol = NULL;
  
  impulse_init(argc, argv, "client");
  
  app_init(&protocol, &local, &remote);
  
  app_main(protocol, local, remote);

  impulse_cleanup();
  #endif
  return 0;
}

#if 0
void app_init(impulse_unistring *arg_protocol,
	      impulse_unistring *arg_local,
	      impulse_unistring *arg_remote)
{
  /*
   * Nb. You would be right in thinking all this creating and destroying
   * of strings just to process command arguments is a little tedious.
   * However once the gettext interface is ready most of this crap
   * will dissapper.
   */
  impulse_unistring pname = impulse_unistring_create_utf("protocol", 8);
  impulse_unistring phelp = impulse_unistring_create_utf("use transport protocol [name]", 8);
  impulse_unistring parg  = impulse_unistring_create_utf("name", 8);
  
  impulse_unistring lname = impulse_unistring_create_utf("local", 8);
  impulse_unistring lhelp = impulse_unistring_create_utf("bind to local address [name]", 8);
  impulse_unistring larg  = impulse_unistring_create_utf("name", 8);
  
  impulse_unistring rname = impulse_unistring_create_utf("remote", 8);
  impulse_unistring rhelp = impulse_unistring_create_utf("connect to remote address [name]", 8);
  impulse_unistring rarg  = impulse_unistring_create_utf("name", 8);
  
  impulse_command_value pvalue = NULL;
  impulse_command_value lvalue = NULL;
  impulse_command_value rvalue = NULL;
  
  impulse_command_option_arg(pname, 0, parg, IMPULSE_COMMAND_TYPE_STRING, phelp);
  impulse_command_option_arg(lname, 0, larg, IMPULSE_COMMAND_TYPE_STRING, lhelp);
  impulse_command_option_arg(rname, 0, rarg, IMPULSE_COMMAND_TYPE_STRING, rhelp);

  impulse_command_parse();
  
  if (impulse_command_set(pname, &pvalue))
    *arg_protocol = impulse_unistring_create(pvalue->s);
  else
    impulse_command_help();
  
  if (impulse_command_set(lname, &lvalue))
    *arg_local = impulse_unistring_create(lvalue->s);
  
  if (impulse_command_set(rname, &rvalue))
    *arg_remote = impulse_unistring_create(rvalue->s);
  else
    impulse_command_help();
  
  impulse_unistring_destroy(pname);
  impulse_unistring_destroy(phelp);
  impulse_unistring_destroy(parg);
  impulse_unistring_destroy(lname);
  impulse_unistring_destroy(lhelp);
  impulse_unistring_destroy(larg);
  impulse_unistring_destroy(rname);
  impulse_unistring_destroy(rhelp);
  impulse_unistring_destroy(rarg);
}


void app_main(impulse_unistring protocol,
	      impulse_unistring local,
	      impulse_unistring remote)
{
  impulse_transport transport = NULL;
  impulse_channel channel = NULL;
  impulse_int ret = 0;
  impulse_bool waiting = 0;
  impulse_unistring alocal = NULL;
  impulse_unistring aremote = NULL;
  

  if (!(transport = impulse_transport_create(protocol)))
  {
    impulse_unistring desc = impulse_unistring_create_utf("Unknown transport protocol requested", 8);
    impulse_error_raise(IMPULSE_ERROR_CRITICAL, desc, protocol);
  }

  impulse_transport_handle(transport, transport_event, &waiting);

  /*
   * First off, resolve the local address (if any)
   */
  if (local &&
      (ret = impulse_transport_lookup(transport, local, &alocal)) < 0)
  {
    if (ret == IMPULSE_TRANSPORT_ADDRESS_RESOLVE)
    {
      waiting = 1;
      while (waiting)
	impulse_event_handle();
      
      ret = impulse_transport_lookup(transport, local, &alocal);
    }

    if (ret)
    {
      impulse_unistring desc = impulse_unistring_create_utf("Address resolution failed", 8);
      impulse_error_raise(IMPULSE_ERROR_CRITICAL,
			  desc,
			  local);
    }
  }

  /*
   * Now we had better do the same for the remote address 
   */
  if ((ret = impulse_transport_lookup(transport, remote, &aremote)) < 0)
  {
    if (ret == IMPULSE_TRANSPORT_ADDRESS_RESOLVE)
    {
      waiting = 1;
      while (waiting)
	impulse_event_handle();
      
      ret = impulse_transport_lookup(transport, remote, &aremote);
    }

    if (ret)
    {
      impulse_unistring desc = impulse_unistring_create_utf("Address resolution failed", 8);
      impulse_error_raise(IMPULSE_ERROR_CRITICAL,
			  desc,
			  remote);
    }
  }

  /*
   * Finally we are ready to connect to the server
   */
  if ((ret = impulse_transport_open(transport,alocal, aremote)) < 0)
  {
    if (ret == IMPULSE_TRANSPORT_CONNECTION_PENDING)
    {
      waiting = 1;
      while (waiting)
	impulse_event_handle();
    } else {
      impulse_unistring desc = impulse_unistring_create_utf("Connection attempt failed", 8);
      impulse_error_raise(IMPULSE_ERROR_CRITICAL,
			  desc,
			  remote);
    }
  }

  channel = impulse_channel_create(transport, 1);
  impulse_channel_register(channel, &service);
  impulse_channel_activate(channel, &service);
  impulse_event_register_fd(STDIN_FILENO, IMPULSE_EVENT_FD_READ, console_event, NULL, channel);

  impulse_main();
  
  impulse_event_deregister_fd(STDIN_FILENO);
  impulse_channel_destroy(channel);
}


impulse_bool console_event(impulse_event *event,
			   void *data)
{  
  impulse_channel channel = (impulse_channel)data;
  impulse_char buffer[1024];
  impulse_int offset = 0;

  for(;;)
  {
    char c;
    
    switch (read(event->data.fd.fd, &c, 1))
    {
     case -1:
      printf("Console read failed\n");
      impulse_quit();
      break;
      
     case 0:
      printf("End of file\n");
      impulse_quit();
      return 0;
    }
    
    buffer[offset++] = c;
    if (offset == 1022)
    {
      c = '\n';
      buffer[offset++] = c;
    }
    if (c == '\n')
    {
      impulse_int length;
      impulse_message message;
      buffer[offset-1] = '\0';
      
      length = 4+strlen(buffer)+1;
      message = impulse_message_create(1,1,length);

      impulse_message_write(message, "xc", length-4, buffer);
      
      impulse_channel_send(channel, message);
      impulse_message_destroy(message);
      break;
    }
  }
  return 1;
}


impulse_bool transport_event(impulse_transport transport,
			     impulse_transport_event event,
			     void *data)
{
  if (event == IMPULSE_TRANSPORT_EVENT_DATA)
  {
    impulse_int pending = impulse_transport_pending(transport);
    impulse_int got = 0;
    impulse_char buffer[1024];
    
    if (pending <= 0)
    {
      printf("Cannot determine pending data %d\n", pending);
      impulse_quit();
      return 0;
    }
    
    /* Scale down bytes read to a manageable size */
    if (pending > 1023)
      pending = 1023;
    
    got = impulse_transport_receive(transport, buffer, pending, NULL);
    if (got <= 0)
    {
      printf("Connection to host lost %d\n", got);
      impulse_quit();
      return 0;
    }
    
    buffer[got] = '\0';
    printf(buffer);

    return 1;
  }
  else if (event == IMPULSE_TRANSPORT_EVENT_CONNECT ||
	   event == IMPULSE_TRANSPORT_EVENT_RESOLVE)
  {
    impulse_bool *waiting = (impulse_bool *)data;
    
    *waiting = 0;
  }
  else
  {
    printf("What the hell is this event doing here ?\n");
  }
  return 0;
}


impulse_bool impulse_service_handler(impulse_channel channel,
				     impulse_channel_service service,
				     impulse_channel_event event)
{
  impulse_message message = &event->data.message;
  impulse_uint length;
  impulse_char *data;
  
  impulse_message_read(message, "xc", &length, &data);
  printf("Receive [%s]\n", data);
  impulse_free(data);
}


#endif
/* End of file */

