
#include <stdio.h>
#include <unistd.h>

#include "impulse/util/main.h"
#include "impulse/util/error.h"
#include "impulse/util/hash.h"
#include "impulse/comms/channel.h"

#if 0
/* 
 * Some function declarations
 */
void app_init(impulse_unistring *,
	      impulse_unistring *);
void app_main(impulse_unistring,
	      impulse_unistring);

impulse_bool transport_event(impulse_transport transport,
			     impulse_transport_event event,
			     void *data);

impulse_bool impulse_service_handler(impulse_channel,
				     impulse_channel_service,
				     impulse_channel_event);

static struct _impulse_unistring service_name = {
  11,11,
  (impulse_unichar[]){
  (impulse_unichar)'T',
  (impulse_unichar)'e',
  (impulse_unichar)'s',
  (impulse_unichar)'t',
  (impulse_unichar)'S',
  (impulse_unichar)'e',
  (impulse_unichar)'r',
  (impulse_unichar)'v',
  (impulse_unichar)'i',
  (impulse_unichar)'c',
  (impulse_unichar)'e',
  (impulse_unichar)'\0'
  }
};

static struct _impulse_channel_service service = 
{
  &service_name,
  1,
  (struct _impulse_channel_version[]) {
    (struct _impulse_channel_version){1,1}
  },
  impulse_service_handler
};

static impulse_hash clients = NULL;
#endif

/*
 * This is where it all starts (as if you didn't know that:)
 */
impulse_int main(impulse_int argc, impulse_char *argv[])
{
  #if 0
  impulse_unistring local = NULL;
  impulse_unistring protocol = NULL;
  
  impulse_init(argc, argv, "server");
  
  app_init(&protocol, &local);
  
  app_main(protocol, local);

  impulse_cleanup();
  #endif
  return 0;
}

#if 0
void app_init(impulse_unistring *arg_protocol,
	      impulse_unistring *arg_local)
{
  /*
   * Nb. You would be right in thinking all this creating and destroying
   * of strings just to process command arguments is a little tedious.
   * However once the gettext interface is ready most of this crap
   * will dissapper.
   */
  impulse_unistring pname = impulse_unistring_create_utf("protocol", 8);
  impulse_unistring phelp = impulse_unistring_create_utf("use transport protocol [name]", 8);
  impulse_unistring parg  = impulse_unistring_create_utf("name", 8);
  
  impulse_unistring lname = impulse_unistring_create_utf("local", 8);
  impulse_unistring lhelp = impulse_unistring_create_utf("bind to local address [name]", 8);
  impulse_unistring larg  = impulse_unistring_create_utf("name", 8);
  
  impulse_command_value pvalue = NULL;
  impulse_command_value lvalue = NULL;
  
  impulse_command_option_arg(pname, 0, parg, IMPULSE_COMMAND_TYPE_STRING, phelp);
  impulse_command_option_arg(lname, 0, larg, IMPULSE_COMMAND_TYPE_STRING, lhelp);

  impulse_command_parse();
  
  if (impulse_command_set(pname, &pvalue))
    *arg_protocol = impulse_unistring_create(pvalue->s);
  else
    impulse_command_help();
  
  if (impulse_command_set(lname, &lvalue))
    *arg_local = impulse_unistring_create(lvalue->s);
  else
    impulse_command_help();
  
  impulse_unistring_destroy(pname);
  impulse_unistring_destroy(phelp);
  impulse_unistring_destroy(parg);
  impulse_unistring_destroy(lname);
  impulse_unistring_destroy(lhelp);
  impulse_unistring_destroy(larg);
}


void app_main(impulse_unistring protocol,
	      impulse_unistring local)
{
  impulse_transport transport = NULL;
  impulse_int ret = 0;
  impulse_bool waiting = 0;
  impulse_unistring address = NULL;
  

  if (!(transport = impulse_transport_create(protocol)))
  {
    impulse_unistring desc = impulse_unistring_create_utf("Unknown transport protocol requested", 8);
    impulse_error_raise(IMPULSE_ERROR_CRITICAL, desc, protocol);
  }

  impulse_transport_handle(transport, transport_event, &waiting);

  /*
   * First off, resolve the local address (if any)
   */
  if ((ret = impulse_transport_lookup(transport, local, &address)) < 0)
  {
    if (ret == IMPULSE_TRANSPORT_ADDRESS_RESOLVE)
    {
      waiting = 1;
      while (waiting)
	impulse_event_handle();
      ret = impulse_transport_lookup(transport, local, &address);
    }

    if (ret)
    {
      impulse_unistring desc = impulse_unistring_create_utf("Address resolution failed", 8);
      impulse_error_raise(IMPULSE_ERROR_CRITICAL,
			  desc,
			  local);
    }
  }
  /*
   * Finally we are ready to connect to the server
   */
  if ((ret = impulse_transport_open(transport,address, NULL)) < 0)
  {
    if (ret == IMPULSE_TRANSPORT_CONNECTION_PENDING)
    {
      waiting = 1;
      while (waiting)
	impulse_event_handle();
    } else {
      impulse_unistring desc = impulse_unistring_create_utf("Connection attempt failed", 8);
      impulse_error_raise(IMPULSE_ERROR_CRITICAL,
			  desc,
			  address);
    }
  }
  impulse_transport_handle(transport, transport_event, transport);

  clients = impulse_hash_create();
  impulse_hash_cleaner(clients, (impulse_hash_cleaner_func)impulse_channel_destroy);
  printf("Start\n");
  impulse_main();
  
  impulse_hash_destroy(clients);
  impulse_transport_destroy(transport);
}

impulse_bool transport_event(impulse_transport transport,
			     impulse_transport_event event,
			     void *data)
{
  if (event == IMPULSE_TRANSPORT_EVENT_INCOMING)
  {
    impulse_transport client = NULL;
    impulse_int ret;
    
    if ((ret = impulse_transport_accept(transport, &client)) < 0)
    {
      /* Error */
    }
    else if (ret)
    {
      impulse_channel channel = impulse_channel_create(client, 0);
      impulse_channel_register(channel, &service);
      impulse_hash_store(clients, client->remote, channel);
      printf("Accepted connection from client\n");
    }
  }
  else if (event == IMPULSE_TRANSPORT_EVENT_RESOLVE)
  {
    impulse_bool *waiting = (impulse_bool *)data;
    *waiting = 0;
  }
  else
  {
    printf("What the hell is this event (%d) doing here ?\n", event);
  }
  return 0;
}


impulse_bool impulse_service_handler(impulse_channel channel,
				     impulse_channel_service service,
				     impulse_channel_event event)
{
  impulse_message message = &event->data.message;
  impulse_message reply;
  impulse_uint length;
  impulse_char *data;
  impulse_iterator iterator = NULL;
  
  impulse_message_read(message, "xc", &length, &data);
  printf("[%s]\n", data);
  impulse_free(data);
  
  iterator = impulse_hash_elements(clients);
  while (!impulse_iterator_eof(iterator))
  {
    impulse_channel channel = (impulse_channel)impulse_iterator_next(iterator);
    
    impulse_channel_send(channel, message);
  }
  impulse_iterator_destroy(iterator);
}
#endif
/* End of file */

