/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 */

/*
 * bitmap.c
 * --------
 *
 * Test that widget set!
 */

#include "impulse/util/main.h"
#include "impulse/util/event.h"
#include "impulse/util/thread.h"
#include "impulse/util/overseer.h"

#include "impulse/rows/main.h"
#include "impulse/rows/window.h"
#include "impulse/rows/colour.h"

#include "bitmap_test.h"
#include "colour_test.h"
//#include "gl_test.h"

impulse_int main(impulse_int argc,
		 impulse_char *argv[])
{
  ROWS_widget window;
  bitmap_test child;
  ROWS_widget window2;
  colour_test child2;
  ROWS_point  size;

  ROWS_init(argc, argv, "bitmap");
  impulse_command_parse();
  impulse_overseer_oversee();

  window = ROWS_window_create("Bitmap test");
  size.x = PIXEL_TO_MICRONS(42);
  size.y = PIXEL_TO_MICRONS(42);
  ROWS_child_set_size(ROWS_child_cast(window),
		      &size);

  child = bitmap_test_create();
  ROWS_constructor_add_child(ROWS_constructor_cast(window),
			     ROWS_child_cast(child),
			     ROWS_CONSTRUCTOR_GL_LEFT,
			     ROWS_CONSTRUCTOR_GL_RIGHT,
			     ROWS_CONSTRUCTOR_GL_TOP,
			     ROWS_CONSTRUCTOR_GL_BOTTOM);
  ROWS_child_set_visible(ROWS_child_cast(child),1);
  ROWS_child_set_visible(ROWS_child_cast(window),1);

  window2 = ROWS_window_create("XYZ colour space");
  size.x = PIXEL_TO_MICRONS(200);
  size.y = PIXEL_TO_MICRONS(200);
  ROWS_child_set_size(ROWS_child_cast(window2),
		      &size);

  child2 = colour_test_create(ROWS_colour_XYZ);
  ROWS_constructor_add_child(ROWS_constructor_cast(window2),
			     ROWS_child_cast(child2),
			     ROWS_CONSTRUCTOR_GL_LEFT,
			     ROWS_CONSTRUCTOR_GL_RIGHT,
			     ROWS_CONSTRUCTOR_GL_TOP,
			     ROWS_CONSTRUCTOR_GL_BOTTOM);
  ROWS_child_set_visible(ROWS_child_cast(child2),1);
  ROWS_child_set_visible(ROWS_child_cast(window2),1);

  window2 = ROWS_window_create("Yxy colour space");
  size.x = PIXEL_TO_MICRONS(200);
  size.y = PIXEL_TO_MICRONS(200);
  ROWS_child_set_size(ROWS_child_cast(window2),
		      &size);
  child2 = colour_test_create(ROWS_colour_Yxy);
  ROWS_constructor_add_child(ROWS_constructor_cast(window2),
			     ROWS_child_cast(child2),
			     ROWS_CONSTRUCTOR_GL_LEFT,
			     ROWS_CONSTRUCTOR_GL_RIGHT,
			     ROWS_CONSTRUCTOR_GL_TOP,
			     ROWS_CONSTRUCTOR_GL_BOTTOM);
  ROWS_child_set_visible(ROWS_child_cast(child2),1);
  ROWS_child_set_visible(ROWS_child_cast(window2),1);

  window2 = ROWS_window_create("Lab colour space");
  size.x = PIXEL_TO_MICRONS(200);
  size.y = PIXEL_TO_MICRONS(200);
  ROWS_child_set_size(ROWS_child_cast(window2),
		      &size);

  child2 = colour_test_create(ROWS_colour_LAB);
  ROWS_constructor_add_child(ROWS_constructor_cast(window2),
			     ROWS_child_cast(child2),
			     ROWS_CONSTRUCTOR_GL_LEFT,
			     ROWS_CONSTRUCTOR_GL_RIGHT,
			     ROWS_CONSTRUCTOR_GL_TOP,
			     ROWS_CONSTRUCTOR_GL_BOTTOM);
  ROWS_child_set_visible(ROWS_child_cast(child2),1);
  ROWS_child_set_visible(ROWS_child_cast(window2),1);

  window2 = ROWS_window_create("Luv colour space");
  size.x = PIXEL_TO_MICRONS(200);
  size.y = PIXEL_TO_MICRONS(200);
  ROWS_child_set_size(ROWS_child_cast(window2),
		      &size);
  child2 = colour_test_create(ROWS_colour_LUV);
  ROWS_constructor_add_child(ROWS_constructor_cast(window2),
			     ROWS_child_cast(child2),
			     ROWS_CONSTRUCTOR_GL_LEFT,
			     ROWS_CONSTRUCTOR_GL_RIGHT,
			     ROWS_CONSTRUCTOR_GL_TOP,
			     ROWS_CONSTRUCTOR_GL_BOTTOM);
  ROWS_child_set_visible(ROWS_child_cast(child2),1);
  ROWS_child_set_visible(ROWS_child_cast(window2),1);

  window2 = ROWS_window_create("Yuv colour space");
  size.x = PIXEL_TO_MICRONS(200);
  size.y = PIXEL_TO_MICRONS(200);
  ROWS_child_set_size(ROWS_child_cast(window2),
		      &size);
  child2 = colour_test_create(ROWS_colour_YUV);
  ROWS_constructor_add_child(ROWS_constructor_cast(window2),
			     ROWS_child_cast(child2),
			     ROWS_CONSTRUCTOR_GL_LEFT,
			     ROWS_CONSTRUCTOR_GL_RIGHT,
			     ROWS_CONSTRUCTOR_GL_TOP,
			     ROWS_CONSTRUCTOR_GL_BOTTOM);
  ROWS_child_set_visible(ROWS_child_cast(child2),1);
  ROWS_child_set_visible(ROWS_child_cast(window2),1);

  window2 = ROWS_window_create("Yu'v' colour space");
  size.x = PIXEL_TO_MICRONS(200);
  size.y = PIXEL_TO_MICRONS(200);
  ROWS_child_set_size(ROWS_child_cast(window2),
		      &size);
  child2 = colour_test_create(ROWS_colour_YUpVp);
  ROWS_constructor_add_child(ROWS_constructor_cast(window2),
			     ROWS_child_cast(child2),
			     ROWS_CONSTRUCTOR_GL_LEFT,
			     ROWS_CONSTRUCTOR_GL_RIGHT,
			     ROWS_CONSTRUCTOR_GL_TOP,
			     ROWS_CONSTRUCTOR_GL_BOTTOM);
  ROWS_child_set_visible(ROWS_child_cast(child2),1);
  ROWS_child_set_visible(ROWS_child_cast(window2),1);

  impulse_main();

  ROWS_cleanup();
  
  return 0;
}
