/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * gl_test.c
 * ---------
 *
 * A test of some graphics
 */

#include "impulse/rows/bitmap.h"
#include "impulse/util/memory.h"
#include "impulse/util/measure.h"

#include "gl_test.h"

/*                            ======--------======                           */

static void gl_test_setup(ROWS_widget *widget);
static int  gl_test_ID   = -1;
static int  gl_test_type = -1;

static void gl_test_destroy(void **private);

static impulse_bool  gl_test_realise(ROWS_child widget,
				     ROWS_child_event_num event_number,
				     ROWS_child_event *event,
				     void *data);
static impulse_bool  gl_test_redraw(ROWS_child widget,
				    ROWS_child_event_num event_number,
				    ROWS_child_event *event,
				    void *data);
static impulse_bool  gl_test_button(ROWS_child widget,
				    ROWS_child_event_num event_number,
				    ROWS_child_event *event,
				    void *data);
static impulse_bool  gl_test_crossing(ROWS_child widget,
				      ROWS_child_event_num event_number,
				      ROWS_child_event *event,
				      void *data);
static impulse_bool gl_test_motion(ROWS_child widget,
				   ROWS_child_event_num event_number,
				   ROWS_child_event *event,
				   void *data);

/*                            ======--------======                           */

/* Standard widget functions */

ROWS_widget gl_test_create(void)
{
  ROWS_widget *wid;

  if (gl_test_ID   == -1)
    gl_test_ID   = ROWS_child_next_identifier();

  wid = gl_test_initialise(gl_test_ID);

  return *wid;
}

ROWS_widget *gl_test_initialise(int id)
{
  ROWS_widget *widget;

  if (gl_test_ID   == -1)
    gl_test_ID   = ROWS_child_next_identifier();
  if (gl_test_type == -1)
    gl_test_type = ROWS_widget_next_type();

  widget = ROWS_child_initialise(id, NULL);
  ROWS_widget_set_type(widget, gl_test_ID, gl_test_type);
  ROWS_widget_set_destructor(widget, gl_test_ID, gl_test_destroy);

  gl_test_setup(widget);

  return widget;
}

int gl_test_next_identifier(void)
{
  if (gl_test_ID==-1)
    gl_test_ID = ROWS_child_next_identifier();

  return gl_test_ID + 1;
}

gl_test gl_test_cast(void **widget)
{
  if (gl_test_ID   == -1)
    gl_test_ID   = ROWS_child_next_identifier();
  if (gl_test_type == -1)
    gl_test_type = ROWS_widget_next_type();

  return ROWS_widget_cast_misc(widget, gl_test_ID, gl_test_type);
}

struct gl_test {
  impulse_bool down;
  ROWS_brush   thick_and_blue;
  ROWS_colour  blue;
};

void gl_test_setup(ROWS_widget *widget)
{
  struct gl_test **private;

  private  = (struct gl_test **) ROWS_widget_private(widget, gl_test_ID);
  *private = impulse_alloc(sizeof(struct gl_test));

  (*private)->down = 0;
  (*private)->thick_and_blue = NULL;
  (*private)->blue = NULL;
   
  ROWS_child_event_register(ROWS_child_cast(*widget),
			    ROWS_CHILD_REALISATION,
			    gl_test_realise,
			    NULL);
 
  ROWS_child_event_register(ROWS_child_cast(*widget),
			    ROWS_CHILD_EXPOSURE,
			    gl_test_redraw,
			    NULL);
  
  ROWS_child_event_register(ROWS_child_cast(*widget),
			    ROWS_CHILD_BUTTON,
			    gl_test_button,
			    NULL);
  ROWS_child_event_register(ROWS_child_cast(*widget),
			    ROWS_CHILD_CROSSING,
			    gl_test_crossing,
			    NULL);
  ROWS_child_event_register(ROWS_child_cast(*widget),
			    ROWS_CHILD_MOTION,
			    gl_test_motion,
			    NULL);
}

void gl_test_destroy(void **private)
{
  /* impulse_free(*private); */
}

     
impulse_bool gl_test_crossing(ROWS_child            widget,
		     ROWS_child_event_num  event_number,
		     ROWS_child_event     *event,
		     void                 *data)
{
  struct gl_test **private;
  
  private  = (struct gl_test **)gl_test_cast(widget);

  /*
   * No need for explicit update cos set activation
   * calls child_update for us :)
   */
  if (event->crossing.entered)
  {
    ROWS_child_set_activation(widget,
			      ROWS_STYLE_HIGHLIGHT);
    if (event->crossing.state & ROWS_STATE_BUTTON_LEFT)
    {
      (*private)->down = 1;
//      ROWS_child_update(widget);
    }
  }
  else
  {
    ROWS_child_set_activation(widget,
			      ROWS_STYLE_NORMAL);
    if (event->crossing.state & ROWS_STATE_BUTTON_LEFT)
    {
      (*private)->down = 0;
//      ROWS_child_update(widget);
    }
  }

  return 0;
}
     
impulse_bool gl_test_button(ROWS_child            widget,
		   ROWS_child_event_num  event_number,
		   ROWS_child_event     *event,
		   void                 *data)
{
  struct gl_test **private;
  
  private  = (struct gl_test **)gl_test_cast(widget);

  if (event->button.pressed)
  {
    if (event->button.button == ROWS_MOUSE_BUTTON_LEFT)
    {
      (*private)->down = 1;
      ROWS_child_update(widget);
    }
  }
  else
  {
    if (event->button.button == ROWS_MOUSE_BUTTON_LEFT)
    {
      (*private)->down = 0;
      ROWS_child_update(widget);
    }
  }

  return 1;
}


impulse_bool gl_test_motion(ROWS_child            widget,
		   ROWS_child_event_num  event_number,
		   ROWS_child_event     *event,
		   void                 *data)
{
  return 1;
}


impulse_bool gl_test_realise(ROWS_child            widget,
			     ROWS_child_event_num  event_number,
			     ROWS_child_event     *event,
			     void                 *data)
{
  ROWS_drawable         drawable;
  struct gl_test      **private;

  private = (struct gl_test **) gl_test_cast(widget);

  drawable = ROWS_drawable_cast(ROWS_child_get_drawable(widget));
  
  if ((*private)->thick_and_blue != NULL)
    {
      ROWS_widget_destroy((*private)->thick_and_blue);
      (*private)->thick_and_blue = NULL;
    }
  if ((*private)->blue != NULL)
    {
      ROWS_colour_destroy((*private)->blue);
      (*private)->blue = NULL;
    }

  if (event->realisation.realised)
    {
      (*private)->thick_and_blue = ROWS_brush_cast(ROWS_brush_create(drawable));
      (*private)->blue = ROWS_colour_rgb_for_drawable(drawable, 0,0,4095);
      ROWS_brush_set_fgcolour((*private)->thick_and_blue,
			      (*private)->blue);
      ROWS_brush_set_linewidth((*private)->thick_and_blue,
			       PIXEL_TO_MICRONS(4));
    }
  
  return 0;
}
     
impulse_bool gl_test_redraw(ROWS_child            widget,
		       ROWS_child_event_num  event_number,
		       ROWS_child_event     *event,
		       void                 *data)
{
  ROWS_style              style;
  ROWS_rect                box;
  ROWS_style_active_type   type;
  struct gl_test      **private;
  impulse_bool down;

  private = (struct gl_test **) gl_test_cast(widget);
  type = ROWS_child_get_activation(widget);

  down = (*private)->down;

  style = ROWS_child_get_style(widget);

  box.size = *ROWS_child_get_size(widget);
  box.pos = *ROWS_child_get_offset(widget);

  {
    ROWS_brush_shape sh[2];

    sh[0].x = box.pos.x + 2000;
    sh[0].y = box.pos.y + 2000;
    sh[1].type = ROWS_BRUSH_BEZIER;
    sh[1].x = box.pos.x + box.size.x - 2000;
    sh[1].y = box.pos.y + box.size.y - 2000;
    sh[1].x1 = box.pos.x;
    sh[1].y1 = box.pos.y + box.size.y;
    sh[1].x2 = box.pos.x + box.size.x;
    sh[1].y2 = box.pos.y;

    ROWS_brush_draw_shape((*private)->thick_and_blue,
			  sh,
			  2);
  }

  ROWS_brush_draw_line(style->brush[type][down ? ROWS_STYLE_BORDERHILITE : ROWS_STYLE_BORDERSHADOW],
		       box.pos.x+box.size.x-PIXEL_MULTIPLIER,
		       box.pos.y,
		       box.pos.x+box.size.x-PIXEL_MULTIPLIER,
		       box.pos.y+box.size.y-PIXEL_MULTIPLIER);
  ROWS_brush_draw_line(style->brush[type][down ? ROWS_STYLE_BORDERHILITE : ROWS_STYLE_BORDERSHADOW],
		       box.pos.x+box.size.x-PIXEL_MULTIPLIER,
		       box.pos.y+box.size.y-PIXEL_MULTIPLIER,
		       box.pos.x,
		       box.pos.y+box.size.y-PIXEL_MULTIPLIER);
  ROWS_brush_draw_line(style->brush[type][down ? ROWS_STYLE_BORDERSHADOW : ROWS_STYLE_BORDERHILITE],
		       box.pos.x, box.pos.y+box.size.y-PIXEL_MULTIPLIER,
		       box.pos.x, box.pos.y);
  ROWS_brush_draw_line(style->brush[type][down ? ROWS_STYLE_BORDERSHADOW : ROWS_STYLE_BORDERHILITE],
		       box.pos.x, box.pos.y,
		       box.pos.x+box.size.x-PIXEL_MULTIPLIER, box.pos.y);

//  ROWS_brush_fill_rectangle(style->brush[type][ROWS_STYLE_BORDERHILITE],
//			    box.pos.x, box.pos.y,
//			    50000, 50000);
  
  return 1;
}
