/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 */

/*
 * test_event.c
 * ------------
 *
 * Test the generic event handling capabilities
 */

#include <stdio.h>
#include <stdlib.h>

#include "impulse/util/thread.h"
#include "impulse/util/event.h"
#include "impulse/util/main.h"

#include <unistd.h>

char rot[] = { '|', '/', '-', '\\'  };
int rotno = 0;

impulse_bool reader(impulse_event *ev, void *data)
{
  char c;  
  if (read(ev->data.fd.fd, &c, 1) != 1)
  {
    impulse_quit();
    return 0;
  }
  
  if (c == 'q')
  {
    impulse_quit();
    return 0;
  }
  
  if (!impulse_is_quit())
  {
    printf("== In: %c %c\n", c, rot[rotno++]);
    if (rotno == 4)
      rotno = 0;
    return 1;
  }
  return 0;
}


#if USE_THREADS
void *writer(void *data)
#else
impulse_bool writer(impulse_event *ev, void *data)
#endif
{
  int fd = (int)data;
#if USE_THREADS
  for(;!impulse_is_quit();)
#endif
  {
    char c = 'A' + 1 + (int)(26.0 * rand()/(RAND_MAX+1.0));
#if USE_THREADS
    impulse_thread_nap((2+(int) (2.0*rand()/(RAND_MAX+1.0))),0);
#endif
    write(fd, &c, 1);
  }
#if USE_THREADS
  return NULL;
#else
  return 1;
#endif
}


impulse_bool timeout(impulse_event *event,
		    void *data)
{
  printf("Drrrrriiinnggg!\n");
  return 1;
}


impulse_int main(impulse_int argc, 
		 impulse_char *argv[])
{
  impulse_int timer;
  impulse_int i;
  impulse_int channels[11][2];
  impulse_int timers[10];
  
  
  impulse_init(argc, argv, "event");
  impulse_thread_init();
  impulse_command_parse();
  
  for (i = 0 ; i < 10 ; i++)
  {
    pipe(channels[i]);
    timers[i] = impulse_event_timer();
#if USE_THREADS
    impulse_thread_create(writer, (void *)channels[i][1]);
#else
    impulse_event_register_ticker(timers[i], 1, 0, writer, (void*)channels[i][1]);
#endif
  }
  channels[10][0] = STDIN_FILENO;

  printf("Reading from standard input and 10 pipes at once\n");
  printf("Oh yes, and acting on random timeouts\n");
  printf("Type 'q' to quit\n");
  for (i = 0 ; i < 11 ; i++)
  {
    impulse_event_register_fd(channels[i][0], IMPULSE_EVENT_FD_READ, reader, NULL, NULL);
  }

  timer = impulse_event_timer();
  impulse_event_register_ticker(timer, 1, 0, timeout, NULL);
  
  impulse_main();
  
  impulse_thread_cleanup();
  impulse_cleanup();
  return 0;
}
