/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 */

/*
 * test_hash.c
 * -----------
 *
 * Test sparse arrays and hash tables
 */

#include <stdio.h>

#include "impulse/util/main.h"
#include "impulse/util/sparse.h"
#include "impulse/util/overseer.h"

impulse_int main(impulse_int argc, 
		 impulse_char *argv[])
{
  impulse_sparse array;
  impulse_sparse sparse;
  impulse_int x,y;

  impulse_init(argc, argv, "sparse"); /* Grr, interface changes */
  impulse_overseer_oversee();
  impulse_command_parse();
  
  array = impulse_sparse_create();

  printf("Testing sparse arrays:\n");
  printf("Storing\n");
  for (x=256;x>=0;x--)
    {
      impulse_sparse_add(array, x*3, (void *)x);
      /* impulse_sparse_add(array, (x*3)+1, (void *)x); */
      printf(".");
      fflush(stdout);
    }
  printf("\nRecovering\n");
  for (x=0;x<=256;x++)
    {
      y = (int) impulse_sparse_recover(array, x*3);
      printf("%i ", y);
    }
  printf("\nDeleting 100->200\n");
  for (x=100;x<=200;x++)
  {
    printf(".");
    fflush(stdout);
    impulse_sparse_add(array,x*3,NULL);
  }
  printf("\nRecovering\n");
  for (x=0;x<=256;x++)
  {
    
          y = (int) impulse_sparse_recover(array, x*3);
          printf("%i ", y);
  }
  
    printf("\n");
  
  impulse_sparse_destroy(array);
  
  sparse = impulse_sparse_create();
  printf("\n  Array created\n");
  impulse_sparse_add(sparse, 48,  (void *) 1);
  impulse_sparse_add(sparse, 124, (void *) 2);
  impulse_sparse_add(sparse, 256, (void *) 3);
  printf("  Three items added\n");
  printf("  Retrieved: %i %i %i %i\n",
         (int) impulse_sparse_recover(sparse, 48),
         (int) impulse_sparse_recover(sparse, 124),
         (int) impulse_sparse_recover(sparse, 256),
         (int) impulse_sparse_recover(sparse, 56));
  impulse_sparse_destroy(sparse);
  printf("  OK\n");

  impulse_cleanup();
  return 0;
}
