/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Daniel Berrange. Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * channel.h
 * ---------
 *
 * Communication channel functions
 */


#ifndef __impulse_comms_channel_h
#define __impulse_comms_channel_h

#include <impulse/comms/transport.h>
#include <impulse/comms/message.h>

/*                            ======--------======                           */

typedef struct _impulse_channel *impulse_channel;
typedef struct _impulse_channel_service *impulse_channel_service;
typedef struct _impulse_channel_event *impulse_channel_event;

typedef impulse_bool (*impulse_channel_handler)(impulse_channel,
						impulse_channel_service,
						impulse_channel_event);

struct _impulse_channel
{
  impulse_transport transport;
  void *internal;
};

struct _impulse_channel_service
{
  impulse_unistring name;
  impulse_int nversion;
/*  impulse_bool auth; */
  struct _impulse_channel_version {
    impulse_ubyte major;
    impulse_ubyte minor;
  } *version;
  impulse_channel_handler handler;
};

struct _impulse_channel_event
{
  enum {
    IMPULSE_CHANNEL_ACTIVATE,
    IMPULSE_CHANNEL_DEACTIVATE,
    IMPULSE_CHANNEL_MESSAGE
  } type;
  union {
    struct {
      struct _impulse_channel_version version;
      impulse_ubyte opcode;
    } activate;
    struct _impulse_message message;
  } data;
};


/*                            ======--------======                           */

impulse_channel impulse_channel_create(impulse_transport transport,
				       impulse_bool initiator);
void impulse_channel_destroy(impulse_channel channel);

impulse_int impulse_channel_send(impulse_channel channel,
				 impulse_message message);

void impulse_channel_register(impulse_channel channel,
			      impulse_channel_service service);
void impulse_channel_deregister(impulse_channel channel,
				impulse_channel_service service);

void impulse_channel_activate(impulse_channel channel,
			      impulse_channel_service service);
void impulse_channel_deactive(impulse_channel channel,
			      impulse_channel_service service);

#endif

/* End of file */
