/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Daniel Berrange. Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * transport_pipe_int.h
 * --------------------
 *
 * Communication transport functions
 * pipeets protocols
 */


#ifndef __impulse_comms_transport_pipe_int_h
#define __impulse_comms_transport_pipe_int_h

#include <impulse/comms/transport_pipe.h>

struct _impulse_transport_pipe
{
  impulse_int in;
  impulse_int out;
  
  impulse_transport_handler handler;
  void *edata;
};

typedef struct _impulse_transport_pipe *impulse_transport_pipe;

/*                            ======--------======                           */

void impulse_transport_pipe_init(void);
void impulse_transport_pipe_cleanup(void);

void impulse_transport_pipe_create(impulse_transport transport);
void impulse_transport_pipe_destroy(impulse_transport transport);

/* Event functions */
impulse_int impulse_transport_pipe_handle(impulse_transport transport,
					  impulse_transport_handler func,
					  void *edata);
impulse_int impulse_transport_pipe_unhandle(impulse_transport transport);


/* Name resolution stuff */

impulse_int impulse_transport_pipe_lookup(impulse_transport transport,
					  impulse_unistring name,
					  impulse_unistring *address);
impulse_int impulse_transport_pipe_rlookup(impulse_transport transport,
					   impulse_unistring address,
					   impulse_unistring *name);

/* Connection functions */
impulse_int impulse_transport_pipe_open(impulse_transport transport,
					impulse_unistring local,
					impulse_unistring remote);
impulse_int impulse_transport_pipe_accept(impulse_transport transport,
					  impulse_transport *client);
impulse_int impulse_transport_pipe_close(impulse_transport transport);
impulse_int impulse_transport_pipe_active(impulse_transport transport);

/* Data transfer functions */
impulse_int impulse_transport_pipe_pending(impulse_transport transport);
impulse_int impulse_transport_pipe_send(impulse_transport transport,
					impulse_byte *buffer,
					impulse_uint size,
					impulse_unistring address);
impulse_int impulse_transport_pipe_receive(impulse_transport transport,
					   impulse_byte *buffer,
					   impulse_uint size,
					   impulse_unistring *address);

#endif

/* End of file */
