/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the ROWS widget library
 * for Impulse.
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter. Portions created by Andrew Hunter
 * are Copyright (C) 1999 Andrew Hunter.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * bitmap_int.h
 * ------------
 *
 * Bitmap widget
 */

#ifndef __impulse_rows_bitmap_int_h
#define __impulse_rows_bitmap_int_h

#include <impulse/rows/bitmap.h>
#include <impulse/util/module.h>

#ifdef __cplusplus
extern "C" {  
#endif

typedef void (*_p_ROWS_bitmap_draw_func)(void *data);

typedef struct _p_ROWS_bitmap_drawable {
  void                     *int_data;
  _p_ROWS_bitmap_draw_func  destroy_func;
  ROWS_drawable             drawable;

  struct _p_ROWS_bitmap_drawable *next;
} _p_ROWS_bitmap_drawable;

typedef struct {
  impulse_int    format;     /* 1, 2, 4, 8, 16 or 32 */
  impulse_ubyte *palette;    /* Gamma correction for 16 or 32-bpp */

  impulse_int width, height; /* Size, in pixels          */
  impulse_int lineskip;      /* Size of a line, in words */

  impulse_uint  *data;       /* Actual data */
  impulse_ubyte *mask;       /* Mask data   */

  impulse_uint   mod_id;     /* Modification identifier */

  ROWS_bitmap_mask_type mask_type;

  _p_ROWS_bitmap_drawable *draw_data;
} _p_ROWS_bitmap_data;

extern impulse_int _p_ROWS_bitmap_ID;
extern impulse_int _p_ROWS_bitmap_type;


struct __p_ROWS_bitmap_handler
{
  ROWS_bitmap_handler handler;
  impulse_module module;
};

typedef struct __p_ROWS_bitmap_handler *_p_ROWS_bitmap_handler;

/*                            ======--------======                           */

extern void *_p_ROWS_bitmap_get_private_data(ROWS_bitmap              bitmap,
					     ROWS_drawable            drawable,
					     impulse_int              size,
					     _p_ROWS_bitmap_draw_func create,
					     _p_ROWS_bitmap_draw_func destroy);

#ifdef __cplusplus
}
#endif

#endif
