/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * main.c
 * ------
 *
 * Initialisation, a few common functions
 */

#include "../../../config.h"

#include "impulse/rows/main.h"
#include "impulse/util/thread.h"

/*                            ======--------======                           */

#if HAVE_GUI_XWIN
#include "impulse/rows/main_xwin.c"
#else
#error "Main module not implemented on this platform"
#endif

/* Error handling */


static impulse_unistring ROWS_error_desc(ROWS_error e)
{
  impulse_unistring message = NULL;
  
  switch(e)
  {
   case ROWS_ERROR_NO_GUI:
    message = impulse_unistring_create_utf("Unable to connect to GUI",8);
    break;
    
   case ROWS_ERROR_INTERNAL:
    message = impulse_unistring_create_utf("Unspecified internal GUI problem", 8);
    break;

   case WIDGET_WRONGTYPE:
    message = impulse_unistring_create_utf("Invalid widget type", 8);
    break;
    
   case WIDGET_INVALIDTYPE:
    message = impulse_unistring_create_utf("Incorrect widget", 8);
    break;
    
   case CONTAINER_RECURSIVE:
    message = impulse_unistring_create_utf("Container can't contain itself", 8);
    break;
    
   case CHILD_NOCONTAINER:
    message = impulse_unistring_create_utf("Contained widget has no container", 8);
    break;
    
   case CONSTRUCTOR_GL_LOOP:
    message = impulse_unistring_create_utf("Unresovable guideline position: loop encountered", 8);
    break;
    
   case CONSTRUCTOR_GL_INVALID:
    message = impulse_unistring_create_utf("Invalid guideline definition", 8);
    break;

   case CONSTRUCTOR_GL_NONEXISTANT:
    message = impulse_unistring_create_utf("Nonexistant guideline referenced", 8);
    break;

   default:
    message = impulse_unistring_create_utf("Unknown error (uh-oh)", 8);
    break;
  }
  return message;
}


static impulse_unistring ROWS_error_msg(ROWS_error e,
					void *data)
{
  impulse_unistring msg = NULL;

  switch (e)
  {
   case ROWS_ERROR_NO_GUI:
   case ROWS_ERROR_INTERNAL:
    if (data)
    {
      msg = impulse_unistring_create_utf((impulse_char *)data, 8);
      impulse_unistring_prepend_utf(msg, "Description: ", 8);
    }
    break;
    
   case WIDGET_INVALIDTYPE:
    if (data)
    {
      msg = impulse_unistring_create_utf((impulse_char *)data, 8);
      impulse_unistring_prepend_utf(msg, "Was expecting a ", 8);
    }
    break;

   default:
    break;
  }
  
  return msg;
}


void ROWS_error_raise(impulse_error_level level,
		      ROWS_error e,
		      impulse_int value, 
		      void *data)
{
  impulse_unistring desc = ROWS_error_desc(e);
  impulse_unistring msg = ROWS_error_msg(e, data);

  impulse_error_raise(level, desc, msg);
  
  if (msg)
    impulse_unistring_destroy(msg);
  if (desc)
    impulse_unistring_destroy(desc);
}


/* End of file */
