/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * geometry.h
 * ----------
 *
 * Data structures and functions to represent geometrical shapes &
 * transformations (well, really convienience functions for clipping
 * and bezier curves...)
 */

#include <stdlib.h>

#include "impulse/util/geometry.h"
#include "impulse/util/measure.h"
#include "impulse/util/memory.h"

void impulse_geom_bezier_point(impulse_geom_bezier *curve,
			       double               t /* 0<=t<=1 */,
			       impulse_geom_point  *result)
{
  double t1;

  t1 = 1-t;
  
  result->x =
    t1*t1*t1*   curve->p[0].x +
    3*t* t1*t1* curve->p[1].x +
    3*t*t* t1*  curve->p[2].x +
    3*          curve->p[3].x;
  result->y =
    t1*t1*t1*   curve->p[0].y +
    3*t* t1*t1* curve->p[1].y +
    3*t*t* t1*  curve->p[2].y +
    3*          curve->p[3].y;
}

void impulse_geom_bezier_delta(impulse_geom_bezier      *curve,
			       double                    delta,
			       impulse_geom_cubic_delta *result)
{
  double a,b,c,d;

  d = curve->p[0].x;
  c =
    3*curve->p[1].x -
    3*curve->p[0].x;
  b =
    3*curve->p[0].x -
    6*curve->p[1].x +
    3*curve->p[2].x;
  a =
    3*curve->p[1].x -
    curve->p[0].x -
    3*curve->p[2].x +
    curve->p[3].x;

  result->x = d;
  result->Dx = a*delta*delta*delta + b*delta*delta + c*delta;
  result->D2x = 6*a*delta*delta*delta + 2*b*delta*delta;
  result->D3x = 6*a*delta*delta*delta;

  d = curve->p[0].y;
  c =
    3*curve->p[1].y -
    3*curve->p[0].y;
  b =
    3*curve->p[0].y -
    6*curve->p[1].y +
    3*curve->p[2].y;
  a =
    3*curve->p[1].y -
    curve->p[0].y -
    3*curve->p[2].y +
    curve->p[3].y;

  result->y = d;
  result->Dy = a*delta*delta*delta + b*delta*delta + c*delta;
  result->D2y = 6*a*delta*delta*delta + 2*b*delta*delta;
  result->D3y = 6*a*delta*delta*delta;
}
