/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * geometry.h
 * ----------
 *
 * Data structures and functions to represent geometrical shapes &
 * transformations (well, really convienience functions for clipping
 * and bezier curves...)
 */

#ifndef __impulse_util_geometry_h
#define __impulse_util_geometry_h

#include "impulse/util/measure.h"

/* Points (FIXME: change ROWS_point...)  */
typedef struct _impulse_geom_point
{
  microns x, y;
} impulse_geom_point;

/* Bezier curves */
typedef struct _impulse_geom_bezier
{
  impulse_geom_point p[4];
} impulse_geom_bezier;

typedef struct _impulse_geom_cubic_delta
{
  microns x,y;

  double Dx,Dy;
  double D2x,D2y;
  double D3x,D3y;
} impulse_geom_cubic_delta;

void impulse_geom_bezier_point(impulse_geom_bezier *curve,
			       double               t,
			       impulse_geom_point  *result);
void impulse_geom_bezier_delta(impulse_geom_bezier      *curve,
			       double                    delta,
			       impulse_geom_cubic_delta *result);

#endif
