/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter. Portions created by Andrew Hunter
 * are Copyright (C) 1999 Andrew Hunter.
 * All Rights Reserved
 *
 * Contributor(s):
 * Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 */

/*
 * overseer.h
 * ----------
 *
 * Debugging system
 */

#include <stdio.h>
#include <stdlib.h>

#include "impulse/util/overseer.h"
#include "impulse/util/memory.h"

#if USE_OVERSEER

static void summarise(void)
{
  fprintf(stderr, "*****\n");
  fprintf(stderr, "***** Impulse overseer summary follows:\n");

#if CHECK_MEMORY_LEAKS
  fprintf(stderr, "      Memory leak checking is ON:\n");
  {
    struct _impulse_mem_stats  stats;
    struct _impulse_mem_list  *next;

    stats = impulse_memory_get_stats();
    next  = impulse_memory_get_allocated();

    fprintf(stderr, "        %iK allocated (%i blocks, %i bytes extra for impulse housekeeping)\n",
	    stats.current/1024,
	    stats.num_blocks,
	    stats.num_blocks*sizeof(impulse_memory_block));
    fprintf(stderr, "        Maximum usage %iK\n", stats.maximum/1024);
    fprintf(stderr, "        Maximum number of blocks %i\n", stats.max_blocks);
    fprintf(stderr, "        Total memory allocated %iK\n", stats.total/1024);
    fprintf(stderr, "        Total memory freed %iK\n", stats.total_freed/1024);
    if (next != NULL)
      {
	fprintf(stderr, "      MEMORY LEAKS DETECTED\n");

	while (next != NULL)
	  {
	    fprintf(stderr, "        %i bytes allocated at %s:%i",
		    next->d->meta->allocate->size,
		    next->d->meta->allocate->source_file,
		    next->d->meta->allocate->line);
	    if (next->d->meta->allocate->comment != NULL)
	      {
		fprintf(stderr, "\n          (%s)", next->d->meta->allocate->comment);
	      }
	    fprintf(stderr, "\n");

	    next = next->next;
	  }
      }
  }
#else
  fprintf(stderr, "      Memory leak checking is OFF\n");  
#endif
}

void impulse_overseer_oversee(void)
{
  fprintf(stderr, "***** Impulse overseer enabled\n");

  atexit(summarise);
}

#endif
