/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * thread_sun.c
 * ------------
 *
 * Impulse threading routines
 * Sun Threads implementation
 */

#include <errno.h>

/*                            ======--------======                           */

#warning "This code has not been tested at all and will probably not even compile (typos)"

/* Thread function defintions */

void impulse_thread_init(void)
{
}


void impulse_thread_cleanup(void)
{ 
}


impulse_thread impulse_thread_create(impulse_thread_function func,
				     void *argument)
{
  thread_t thr;
  THREAD_ERROR(thr_create(NULL, 0, func, argument, 0, &thr));
  
  return (impulse_thread)thr;
}


impulse_thread impulse_thread_create_detach(impulse_thread_function func,
					    void *argument)
{
  thread_t thr;
  THREAD_ERROR(thr_create(NULL, 0, func, argument, THR_DETACHED, &thr));
  
  return (impulse_thread)thr;
}


void impulse_thread_exit(void *status)
{
  THREAD_ERROR(thr_exit(status));
}


void impulse_thread_join(impulse_thread t, void **status)
{
  THREAD_ERROR(thr_join(t, NULL, status));
}


impulse_thread impulse_thread_self(void)
{
  return (impulse_thread)thr_self();
}


impulse_int impulse_thread_same(impulse_thread a,
				impulse_thread b)
{
  return a == b;
}


void impulse_thread_nap(impulse_int secs,
			impulse_int usecs)
{
  /*
   * It may seem more logical to use sleep() here. However, sleep() is 
   * not thread-safe under most implementations of POSIX threads due
   * to the implementation of signals (the wake-up alarm signal can go 
   * to any thread, having very weird consequences). So, we use
   * select() instead, which is thread-safe usually.
   */
  struct timeval wakeup;

  wakeup.tv_sec  = secs;
  wakeup.tv_usec = usecs;
  
  THREAD_ERROR(select(0, NULL, NULL, NULL, &wakeup));
}


void impulse_thread_yield(void)
{
  THREAD_ERROR(thr_yield());
}


/* Mutex functions */


impulse_mutex impulse_mutex_create(void)
{
  mutex_t *mutex = impulse_alloc(sizeof(mutex_t));
  
  THREAD_ERROR(mutex_init((mutex_t*)mutex), USYNC_THREAD, NULL));
  
  return mutex;
}


impulse_mutex impulse_mutex_create_recursive(void)
{
  mutex_t *mutex = impulse_alloc(sizeof(mutex_t));
  
  THREAD_ERROR(mutex_init((mutex_t*)mutex), USYNC_THREAD, NULL));
  
  return mutex;
}


void impulse_mutex_lock(impulse_mutex mutex)
{
  THREAD_ERROR(mutex_lock((mutex_t*)mutex));
}


impulse_int impulse_mutex_trylock(impulse_mutex mutex)
{
  impulse_int ret;
  ret = mutex_trylock((mutex_t*)mutex);
  if (ret == EBUSY)
    return 0;
  else if (ret < 0)
    THREAD_ERROR(ret);
  return 1;
}


void impulse_mutex_unlock(impulse_mutex mutex)
{
  THREAD_ERROR(mutex_unlock((mutex_t*)mutex));
}


void impulse_mutex_destroy(impulse_mutex mutex)
{
  mutex_destroy((mutex_t*)mutex);
  impulse_free(mutex);
}


static mutex_t global_lock = DEFAULT_MUTEX;

void impulse_mutex_f_global_lock(void)
{
  THREAD_ERROR(mutex_lock(&global_lock));
}

void impulse_mutex_f_global_unlock(void)
{
  THREAD_ERROR(mutex_unlock(&global_lock));
}


/* Condition variable functions */

impulse_cond impulse_cond_create(void)
{
  impulse_cond *cond = impulse_alloc(sizeof(cond_t));
  
  THREAD_ERROR(cond_init((cond_t*)cond, USYNC_THREAD, NULL));
  
  return cond;
}


void impulse_cond_signal(impulse_cond cond)
{
  THREAD_ERROR(cond_signal(cond_t *)cond));
}


void impulse_cond_broadcast(impulse_cond cond)
{
  THREAD_ERROR(cond_broadcast((cond_t *)cond));
}


void impulse_cond_wait(impulse_cond cond,
		       impulse_mutex mutex)
{
  THREAD_ERROR(cond_wait((cond_t*)cond, (mutex_t*)mutex));
}

void impulse_cond_destroy(impulse_cond cond)
{
  THREAD_ERROR(cond_destroy((cond_t*)cond));
  impulse_free(cond);
}


/* Read/write mutexes */

impulse_rwlock impulse_rwlock_create(void)
{
  impulse_rwlock lock = impulse_alloc(sizeof(rwlock_t));
  
  THREAD_ERROR(rwlock_init((rwlock_t*)lock, USYNC_THREAD, NULL));
  
  return lock;
}


void impulse_rwlock_read(impulse_rwlock lock)
{
  THREAD_ERROR(rwlock_rdlock((rwlock_t*)lock));
}


impulse_bool impulse_rwlock_tryread(impulse_rwlock lock)
{
  int ret;
  ret = rwlock_tryrdlock((rwlock_t*)lock);
  if (ret == EBUSY)
    return 0;
  else if (ret < 0)
    THREAD_ERROR(ret);
  return 1;
}


void impulse_rwlock_write(impulse_rwlock lock)
{
  THREAD_ERROR(rwlock_wrlock((rwlock_t*)lock));
}


impulse_bool impulse_rwlock_trywrite(impulse_rwlock lock)
{
  int ret;
  ret = rwlock_trywrlock((rwlock_t*)lock);
  if (ret == EBUSY)
    return 0;
  else if (ret < 0)
    THREAD_ERROR(ret);
  return 1;
}


void impulse_rwlock_unlock(impulse_rwlock lock)
{
  THREAD_ERROR(rwlock_unlock((rwlock_t*)lock));
}


void impulse_rwlock_destroy(impulse_rwlock lock)
{
  THREAD_ERROR(rwlock_destroy((rwlock_t*)lock));
  impulse_free(lock);
}


/* Semaphores */


impulse_sema impulse_sema_create(impulse_int value)
{
  impulse_sema sem = impulse_alloc(sizeof(sema_t));
  
  THREAD_ERROR(sema_init((sema_t*)sem, value, USYNC_THREAD, NULL));
  
  return sem;
}


void impulse_sema_wait(impulse_sema sem)
{
  THREAD_ERROR(sema_wait((sema_t*)sem));
}


impulse_bool impulse_sema_trywait(impulse_sema sem)
{
  int ret;
  ret = sema_trywait((sema_t*)sem);
  if (ret == EBUSY)
    return 0;
  else if (ret < 0)
    THREAD_ERROR(ret);
  return 1;
}


void impulse_sema_post(impulse_sema sem)
{
  THREAD_ERROR(sema_post((sema_t*)sem));
}


void impulse_sema_destroy(impulse_sema sem)
{
  THREAD_ERROR(sema_destroy((sema_t*)sem));
  impulse_free(sem);
}


/* End of file */
