/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * unidata.h
 * ---------
 *
 * Data structures used to represent the Unicode Character Database
 * (www.unicode.org), needed for correct manipulation of Unicode
 * characters. Also structures for representing conversion tables
 * between Unicode and arbitary 
 */

#ifndef __impulse_util_unidata_h
#define __impulse_util_unidata_h

#include <impulse/util/unichar.h>

#ifdef __cplusplus
extern "C" {  
#endif

/*                            ======--------======                           */

/* Unidata structures */

/*
 * FIXME: as there is apparently no provision for surrogate pairs in
 * the Unicode Character Database file format, no provision is made
 * here.
 */

/*
 * Fields marked as optional may not be stored for reasons of space or 
 * omission from the Unicode Character Database. Omitted fields will
 * be NULL (the pointer value or the Unicode value [U+0000]).
 */

/*
 * I've done my best to minimize stored database size, but I assume
 * gcc semantics when it comes to packing structures (gcc could
 * definately be improved in this area - it should automatically
 * rearrange structures to put similar typed elements together). Even
 * so, you can expect a 450k+ object file with the full Unicode
 * database. Not good for static linking ;-)
 */

/*
 * With any luck I've made the database a bit smaller, by introducing
 * bitfields for alot of the categories. Alot of the flags have been
 * renumbered to take advantage of them being mutually exclusive.
 * 
 * Can a letter have both, upper and lower case mappings? If not,
 * we can reduce the size by compressing the three ?case fields
 * into one.
 * 
 * The decimal value is now stored in the numberic_value field,
 * as they appear to either be the same, or mutually exclusive.
 * 
 *        - DPB 23/2/00
 */
  
typedef struct {
  impulse_uint category:5;
  impulse_uint mirrored:1;
  impulse_uint direction:4;
  impulse_uint formatting:5;
  impulse_uint combining:8;
  
  impulse_uint ndecomposition:8;
  impulse_unichar *decomposition;
  
  impulse_unichar uppercase;
  impulse_unichar lowercase;
  impulse_unichar titlecase;
  
  impulse_ubyte digit;
  impulse_float value;
} impulse_unidata_item;

/*                            ======--------======                           */

/*
 * Encoded form of Unicode Character Data (256 arrays of 256 entries
 * each)
 */

extern impulse_unidata_item *impulse_unidata[];

/*                            ======--------======                           */

/* Composition table structures */

#define IMPULSE_UNIDATA_COMPO_BINARY 1
#define IMPULSE_UNIDATA_COMPO_MULTI  2
#define IMPULSE_UNIDATA_COMPO_UNUSED 4

typedef struct _impulse_unidata_composition
{
  impulse_ubyte flags;     /* Flags */
  impulse_unichar composition;

  int number_multis;
  impulse_unichar   *multi_compo;
  struct _multi_data
  {
    impulse_ubyte length;
    impulse_unichar *string;
  } *multi;
} impulse_unidata_composition;

/* Hey! That's a lot of asterisks! (15, in fact) */
extern impulse_unidata_composition ***************impulse_unidata_comp_table[4];

#ifdef __cplusplus
}
#endif

#endif
