dnl 
dnl Macros to determine what comms libraries available
dnl


dnl *****************************************************************
dnl * Initialise configuration
dnl *****************************************************************

AC_DEFUN(IMPULSE_COMMS_INIT,
[
  COMMS_MODULES=""
])


dnl *****************************************************************
dnl * Check to see if thread safe versions are available
dnl *****************************************************************

AC_DEFUN(IMPULSE_COMMS_CHECK_SOCK,
[
  IMPULSE_PARAMS_SAVE
  SOCK_FAIL="0"
  
  dnl
  dnl Assume that if this works then we also have setsockopt,
  dnl accept, bind, connect, listen, getsockname, getpeername, etc.
  dnl We might need to test for recv/send separately ??
  dnl
  AC_CHECK_HEADER(sys/socket.h,
  [
    AC_CHECK_FUNC(socket,
    [
    ],[
      AC_CHECK_LIB(socket, socket,
      [
	COMMS_LIBS="$COMMS_LIBS -lsocket"
      ],[
        SOCK_FAIL="1"
      ])
    ])
  ])
  
  dnl
  dnl At minimum we need gethostbyname and gethostbyaddr.
  dnl If we have gethostbyname2 or any _r variants we will
  dnl be happy. 
  dnl If we have getipnodebyaddr and getipnodebyname we
  dnl will be we will be very happy.
  dnl Don't care about getaddrinfo anymore
  dnl
  if test "$SOCK_FAIL" = "0" ; then
    AC_CHECK_HEADER(netdb.h,
    [
      AC_CHECK_FUNC(gethostbyname,
      [
        AC_CHECK_FUNC(gethostbyaddr,
	[
          AC_DEFINE(HAVE_COMMS_SOCK, 1)
	  
          AC_CHECK_FUNC(gethostbyname2)
	  AC_CHECK_FUNC(gethostbyaddr_r)
	  AC_CHECK_FUNC(gethostbyname2_r)
	  
	  AC_CHECK_FUNC(getipnodebyname)
	  AC_CHECK_FUNC(getipnodebyaddr)
	  AC_CHECK_FUNC(freehostent)
	])
      ],[
        SOCK_FAIL="1"
      ])
    ])
  fi

  if test "$SOCK_FAIL" = "0" ; then
    AC_CHECK_HEADER(sys/ioctl.h,
    [
      AC_CHECK_FUNC(ioctl,
      [
        AC_TRY_COMPILE(
	[
	#include <sys/ioctl.h>
	],[
          ioctl(0, FIONREAD, 0);
	],[
          AC_DEFINE(HAVE_COMMS_SOCK_NREAD, 1)
        ])
      ])
    ])
    
    COMMS_MODULES="$COMMS_MODULES sockets"
    IMPULSE_COMMS_CHECK_SOCK_PROTO
  fi

  IMPULSE_PARAMS_RESTORE
])


dnl *****************************************************************
dnl * Check to see which protocols are available
dnl *****************************************************************

AC_DEFUN(IMPULSE_COMMS_CHECK_SOCK_PROTO,
[
  IMPULSE_PARAMS_SAVE
  
  AC_CHECK_HEADER(netinet/in.h,
  [
    AC_CACHE_CHECK([for IPv4 sockets], impulse_cv_ipv4,
    [
      AC_TRY_COMPILE([
        #include "netinet/in.h"
        #include "sys/socket.h"
      ],[
        struct sockaddr_in addr;
        socket(PF_INET, 0, 0);
      ],[
        impulse_cv_ipv4="yes"
      ],[
        impulse_cv_ipv4="no"
      ])
    ])
  
    AC_CACHE_CHECK([for IPv6 sockets], impulse_cv_ipv6,
    [
      AC_TRY_COMPILE([
        #include "netinet/in.h"
        #include "sys/socket.h"
      ],[
        struct sockaddr_in6 addr;
        socket(PF_INET6, 0, 0);
      ],[
        impulse_cv_ipv6="yes"
      ],[
        impulse_cv_ipv6="no"
      ])
    ])
  ])

  dnl Check for local
  AC_CHECK_HEADER(sys/un.h,
  [
    AC_CACHE_CHECK([for local domain sockets], impulse_cv_local,
    [
      AC_TRY_COMPILE([
        #include "sys/un.h"
        #include "sys/socket.h"
      ],[
        struct sockaddr_un addr;
        socket(PF_LOCAL, 0, 0);
      ],[
        impulse_cv_local="yes"
      ],[
        impulse_cv_local="no"
      ])
    ])
    AC_CACHE_CHECK([for unix domain sockets], impulse_cv_unix,
    [
      AC_TRY_COMPILE([
        #include "sys/un.h"
	#include "sys/socket.h"
      ],[
        struct sockaddr_un addr;
	socket(PF_UNIX, 0, 0);
      ],[
	impulse_cv_unix="yes"
      ],[
        impulse_cv_unix="no"
      ])
    ])
  ])
  
  if test "$impulse_cv_ipv4" = "yes" ; then
    AC_DEFINE(HAVE_COMMS_SOCK_IPV4, 1)
  fi
  if test "$impulse_cv_ipv6" = "yes" ; then
    AC_DEFINE(HAVE_COMMS_SOCK_IPV6, 1)
  fi
  if test "$impulse_cv_local" = "yes" ; then
    AC_DEFINE(HAVE_COMMS_SOCK_LOCAL, 1)
  fi
  if test "$impulse_cv_unix" = "yes" ; then
    AC_DEFINE(HAVE_COMMS_SOCK_UNIX, 1)
  fi
  IMPULSE_PARAMS_RESTORE
])


dnl *****************************************************************
dnl * Check to see what COMMS libraries are available
dnl *****************************************************************

AC_DEFUN(IMPULSE_COMMS_CHECK,
[
  IMPULSE_MSG_SECTION([Checking communications libraries])
  
  IMPULSE_COMMS_CHECK_SOCK
])



dnl *****************************************************************
dnl * Finish up configuration stuff
dnl *****************************************************************


AC_DEFUN(IMPULSE_COMMS_FINISH,
[
  AC_SUBST(COMMS_CFLAGS)
  AC_SUBST(COMMS_LIBS)
  AC_SUBST(COMMS_LDFLAGS)
])


dnl *****************************************************************
dnl * Print configuration summary
dnl *****************************************************************


AC_DEFUN(IMPULSE_COMMS_SUMMARY,
[
  IMPULSE_ECHO(["Communications modules: $COMMS_MODULES"])
])


dnl End of file
